/*
    as5045.c - Magnetic encoder
    Copyright (C) 2018 Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/

#include <libziaint.h>
#include <zas5045.h>

#include <zdebug.h>
#include <zgpio.h>
#include <zthread.h>

#include <glib.h> 
#ifdef Z_HAVE_UNISTD_H
#include <unistd.h>
#endif

struct zas5045 *zas5045_init(char *csgpioname, char *clkgpioname, char *datagpioname)
{
	struct zas5045 *as = g_new0(struct zas5045, 1);

	as->cs = zgpio_init_name(csgpioname);
	if (!as->cs) { error("Can't alloc GPIO %s", csgpioname); goto x; }

	as->clk = zgpio_init_name(clkgpioname);
	if (!as->clk) { error("Can't alloc GPIO %s", clkgpioname); goto x; }

	as->data = zgpio_init_name(datagpioname);
	if (!as->data) { error("Can't alloc GPIO %s", datagpioname); goto x; }


	zgpio_dir_output(as->cs);
	zgpio_dir_output(as->clk);
	zgpio_dir_input(as->data);

	return as;
x:;
	zas5045_free(as);
	return NULL;
}

void zas5045_free(struct zas5045 *as)
{
	if (!as) return;

	if (as->cs != NULL) zgpio_free(as->cs);
	if (as->clk != NULL) zgpio_free(as->clk);
	if (as->data != NULL) zgpio_free(as->data);
	g_free(as);
}

int zas5045_read(struct zas5045 *as){
	int i = 0, d, par;
	unsigned int raw = 0;
	unsigned n;

	zgpio_write(as->clk, 1);
	zgpio_write(as->cs, 0);
	usleep(1); // 500ns

	raw = 0;
    par = 0;
	for (i = 0; i < 18; i++){
		zgpio_write(as->clk, 0);
		usleep(1); // 500 ns
		zgpio_write(as->clk, 1);
		usleep(1); // 413 ns
		raw <<= 1;
		d = zgpio_read(as->data);
        if (d < 0) {
	        zgpio_write(as->cs, 1);
        	usleep(1); // 500 ns
            return ZAS5045_EREAD;
        }
		if (d > 0) {
			raw |= 1;
			par = !par;
		}
	}
	zgpio_write(as->cs, 1);
    
    /*printf("%04x  ", raw);
    for (i = 17; i >= 0; i--){
        printf("%d", (raw & (1<<i)) != 0 ? 1 : 0);
    }
    printf("\n");
    return 0;    */

	usleep(1); // 500 ns

	n = raw >> 6;

	/*	printf("raw=%06x  n=%04d  ocf=%d cof=%d ln=%d inc=%d dec=%d p=%d par=%d\n", 
				raw, n,
				!!(raw & 32), !!(raw & 16), !!(raw & 8), !!(raw & 4), !!(raw & 2), !!(raw & 1),
				par);*/
      
	if (par != 0){
		return ZAS5045_EPARITY;
	}

	as->ocf = (raw & 32) != 0 ? 1 : 0;
	as->cof = (raw & 16) != 0 ? 1 : 0;
	as->lin = (raw & 8) != 0 ? 1 : 0;
	as->maginc = (raw & 4) != 0 ? 1 : 0;
	as->magdec = (raw & 2) != 0 ? 1 : 0;

	if (as->cof) {
		return ZAS5045_ECOF;
	}

	return (int)n;
}


int zas5045_read_azimuth(struct zas5045 *as){
	int ret = zas5045_read(as);
	if (ret < 0) return ret;

	ret = (ret * 360) / 4096;
	ret = 90 - ret + 360;
	ret = ret % 360;

	return ret;
}
